#+BEGIN_COMMENT
.. title: Experience with Python...
.. slug: experience-with-python
.. date: 2018-10-10 11:10:07 UTC+05:30
.. tags: programming, python, reflections
.. category: reflections
.. link: 
.. description: Even after working with python for long time, how comfortable I am with it.
.. type: text
#+END_COMMENT


Last weekend I attended [[https://in.pycon.org/2018/][10th Edition of PyCon India]] in Hyderabad. I
had started using python seriously around 2009 when [[https://in.pycon.org/2009/][first PyCon India]]
happened. During first edition entire [[https://fossee.in/][FOSSEE]] team was participating
and while I was getting better understanding of using Python we as
team were also conducting introductory workshops around it. We were
pitching python really hard, our [[https://hg.fossee.in/workshops/][workshops content]] had all cool
features which python was offering and how coding becomes so easy
compared to Matlab or C or other languages being used in Indian
colleges. Back then I personally also believed in what I was
preaching, but today, I have my doubts.

In latest Pycon edition, opening keynote was given by [[https://github.com/mitsuhiko][Armin]], and his
talk was around future of Python, given that [[https://mail.python.org/pipermail/python-committers/2018-July/005664.html][Guido is removing himself
from decision process]]. One point which was covered in the talk was
about how there are guidelines via certain PEPs, but language by
itself never enforces them. I have personally used [[https://gitlab.com/pycqa/flake8][flake8]] to adhere to
a certain formatting, but again, its an external tool and still not
part of standard libraries. The talk, bubbled up some of my recent
doubts on what is right approach of doing something. During first
PyCon, if I remember correctly, we had zen of python printed on the
TShirts which we were handing out, and I very clearly remembered the
line:

#+BEGIN_QUOTE
There should be one-- and preferably only one --obvious way to do it.
#+END_QUOTE

Here is a small example on how ~try~, ~except~ leaves me confused on
what would be right approach to use them. I am quoting from a
[[https://stackoverflow.com/a/16138864][StackOverflow conversation]], which goes like:

#+BEGIN_QUOTE
In the Python world, using exceptions for flow control is common and
normal.
#+END_QUOTE

So, I have a dictionary and I want to use one of its key and I can do
something like:

#+BEGIN_SRC python
try:
    uri = station['uri']
except KeyError:
    return
#+END_SRC

python ~dict~ supports a ~get~ which can return default value in case
key is missing. So using that we can also handle above situation using
a ~if~ ~else~ block:

#+BEGIN_SRC python
if station.get('uri'):
    uri = station.get('uri')
else:
    return

#################### OR ####################
uri = station.get('uri')
if not uri:
    return

#################### OR ####################
if 'uri' in station:
    uri = station['uri']
else:
    return
#+END_SRC

I already feel a bit lost or conflicted, if I follow quote from
stackoverflow, first method is and should be the way things need to be
done. But I have used both methods in different situation, and that is
conflicting to the zen ideology quoted above.

Furthermore, I have tried to use ~try~, ~except~ and limit the scope
of the block to minimum lines of code instead of stretching it to
complete function or logic, but there, handling ~return~ for every
exception, made the code hard to follow. For multiple exit points I
make sure that at every exit, number of variables returned are same, I
write ~unittests~ to confirm that it is the case, all in place, tests
are passing, and at the end, code doesn't look clean anymore, I feel
that I am violating the first line of zen:

#+BEGIN_QUOTE
Beautiful is better than ugly.
#+END_QUOTE

Personally, I think there is lot of room to get better at this, these
are very fundamental concepts which shouldn't be hard or
confusing. From here on, I will be looking out for some pattern across
popular libraries and their implementation, understand their approach
and get clarity on which methodology is better suited for which
situation.
